#ifndef EQUVER_H
#define EQUVER_H
#define MAXVARS 1024
#define MAXEQUS 1024

typedef double bif_t(double in);
typedef double bif2_t(double in1, double in2);

typedef enum operator_e {
	OP_ADD,
	OP_SUB,
	OP_MUL,
	OP_DIV
} operator_t;

typedef enum opcode_e {
	OPC_OP,
	OPC_NUM,
	OPC_VAR,
	OPC_BIF,
	OPC_BIF2
} opcode_t;

typedef struct code_s code_t;

struct code_s {
	opcode_t c;
	union {
		operator_t op;
		double num;
		int var;
		bif_t *bif;
		bif2_t *bif2;
	} data;
	code_t *next;
};

typedef enum {
	VF_NONE          = 0,
	VF_SILENT        = 1,
	VF_SCALE_SUFFIX  = 2,
	VF_CONSTANT      = 4
} vflags_t;

code_t *alloc_bif(opcode_t code, bif_t *bif, code_t *parm1, code_t *parm2);
code_t *alloc_op(operator_t op, code_t *c1, code_t *c2);
code_t *alloc_num(double num);
code_t *alloc_var(const char *vname, vflags_t orflags);
void fin_eq(code_t *code);
void swap_eq(int e1, int e2);
void swap_vars(int v1, int v2);
extern int verbose;


/* append function text from lex */
void append(const char *s);
void append_reset();

/* for internal use (optimization) */
double eq_eval(code_t *eq, double *v);
void print_code(code_t *code);

/* for bison */
int yyparse ();

#endif /* EQUVER_H */
