/*  equver - solver of non-linear system of equations given in infix form
    Copyright (C) 2009  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page: http://repo.hu/projects/equver
    Contact:      equver@igor2.repo.hu
*/

%{
#include <stdlib.h>
#include <stdio.h>
#include "equver.h"

static char *yy_vname;

%}

%union {
	char *str;
	double num;
	void *bif;
	void *code;
}

%locations
%error-verbose

%token <str> ID
%token <num> NUM
%token <str> VERBOSE
%token <bif> BIF;
%token <bif> BIF2;

%type <code> expr

%left '-' '+' 
%left '*' '/'
%left NL

%%

/* tolerate newlines (empty lines) before the equations */
eqsys:
	  nls equation
	| equation
	;

equation:
	  eqline nls equation
	| init nls equation
	| /* empty */
	;

nls:
	  NL
	| nls NL
	;

init:
	  ID ':' '=' NUM       { set_initial($1, $4); free($1); append_reset(); }
	| ID ':' '=' '-' NUM   { set_initial($1, -$5); free($1); append_reset(); }
	| VERBOSE ':' '=' NUM  { verbose = $4; append_reset(); }
	| ID '%' '='
	            { yy_vname = $1; } vflags { append_reset(); }
	;

vflags:
	  ID                   { var_set_flag(yy_vname, $1); }
	| ID vflags            { var_set_flag(yy_vname, $1); }
	;

eqline:
	expr '=' expr { fin_eq(alloc_op(OP_SUB, $3, $1));}
	;

expr:
	  NUM                            { $$ = alloc_num($1); }
	| BIF '(' expr ')'               { $$ = alloc_bif(OPC_BIF, $1, $3, NULL); }
	| BIF2 '(' expr ',' expr ')'     { $$ = alloc_bif(OPC_BIF2, $1, $3, $5); }
	| '-' expr                       { $$ = alloc_op(OP_SUB, $2, alloc_num(0)); }
	| ID                             { $$ = alloc_var($1, VF_NONE); free($1); }
	| NUM ID                         { $$ = alloc_op(OP_MUL, alloc_num($1), alloc_var($2, VF_NONE)); free($2); }
	| ID ID                          { $$ = alloc_op(OP_MUL, alloc_var($1, VF_NONE), alloc_var($2, VF_NONE)); free($1); free($2); }
	| expr '+' expr                  { $$ = alloc_op(OP_ADD, $3, $1); } 
	| expr '-' expr                  { $$ = alloc_op(OP_SUB, $3, $1); }
	| expr '/' expr                  { $$ = alloc_op(OP_DIV, $3, $1); }
	| expr '*' expr                  { $$ = alloc_op(OP_MUL, $3, $1); }
	| '(' expr ')'                   { $$ = $2; }
	;
