/*  equver - solver of non-linear system of equations given in infix form
    Copyright (C) 2015  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page: http://repo.hu/projects/equver
    Contact:      equver@igor2.repo.hu
*/

#include <stdlib.h>
#include <string.h>
#include "ipath.h"

#define PATHMX 8192
typedef struct ipath_s ipath_t;

struct ipath_s {
	const char *path;
	int len;
	ipath_t *next;
};

ipath_t *ipaths;

void ipath_insert(const char *path)
{
	ipath_t *i;
	int l;

	l = strlen(path);
	if (l > PATHMX - 2) {
		fprintf(stderr, "-I path too long: '%s'\n", path);
		abort();
	}

	i = malloc(sizeof(ipath_t));
	i->path = path;
	i->len  = l;
	i->next = ipaths;
	ipaths = i;
}

FILE *ipath_open(const char *fn)
{
	FILE *f;
	ipath_t *i;
	int l;
	char fp[PATHMX];

	if ((fn[0] == '/') || ((fn[0] == '.') && (fn[1] == '/')) || ((fn[0] == '.') && (fn[1] == '.') && (fn[2] == '/')))
		return fopen(fn, "r");

	l = strlen(fn);
	for(i = ipaths; i != NULL; i = i->next) {
		if (l + i->len > PATHMX-3) {
			fprintf(stderr, "Warning: include path+filename got too long\n");
			continue;
		}
		memcpy(fp, i->path, i->len);
		fp[i->len] = '/';
		memcpy(fp+i->len+1, fn, l+1);
		f = fopen(fp, "r");
		if (f != NULL)
			return f;
	}
	return NULL;
}

void ipath_reset(void)
{
	ipaths = NULL;
}
